/*
---

script: IframeShim.js

name: IframeShim

description: Defines IframeShim, a class for obscuring select lists and flash objects in IE.

license: MIT-style license

authors:
  - Aaron Newton

requires:
  - Core/Element.Event
  - Core/Element.Style
  - Core/Options
  - Core/Events
  - /Element.Position
  - /Class.Occlude

provides: [IframeShim]

...
*/

var IframeShim = new Class({

	Implements: [Options, Events, Class.Occlude],

	options: {
		className: 'iframeShim',
		src: '',
		display: false,
		zIndex: null,
		margin: 0,
		offset: {x: 0, y: 0},
		browsers: (Browser.ie6 || (Browser.firefox && Browser.version < 3 && Browser.Platform.mac))
	},

	property: 'IframeShim',

	initialize: function(element, options){
		this.element = document.id(element);
		if(this.element){
			if (this.occlude()) return this.occluded;
		}
		this.setOptions(options);
		this.makeShim();
		return this;
	},

	makeShim: function(){
		if (this.options.browsers){
			if(this.element){
				var zIndex = this.element.getStyle('zIndex').toInt();
	
				if (!zIndex){
					zIndex = 1;
					var pos = this.element.getStyle('position');
					if (pos == 'static' || !pos) this.element.setStyle('position', 'relative');
					this.element.setStyle('zIndex', zIndex);
				}
				zIndex = ((this.options.zIndex != null || this.options.zIndex === 0) && zIndex > this.options.zIndex) ? this.options.zIndex : zIndex - 1;
				
				//ΪMUI.ModalʱID_canvas _overlay
				if(this.element.getAttribute('id').search(/_canvas/) != '-1'){
					$(this.element.getAttribute('id').slice(0,-7)+'_overlay').setStyle('zIndex','2');  //firefox½޷ʾdivڵ
				}
				if (zIndex < 0) zIndex = 1;
				this.shim = new Element('iframe', {
					src: this.options.src,
					id: this.element.getAttribute('id')+'_iframeshim',   //ID
					scrolling: 'no',
					frameborder: 0,
					styles: {
						zIndex: zIndex,
						position: 'absolute',
						//opacity: 0.1,   //͸
						border: 'none',
						filter: 'progid:DXImageTransform.Microsoft.Alpha(style=0,opacity=0)'
					},
					'class': this.options.className
				}).store('IframeShim', this);
				var inject = (function(){
					this.shim.inject(this.element, 'after');
					this[this.options.display ? 'show' : 'hide']();
					this.fireEvent('inject');
				}).bind(this);
				if (!IframeShim.ready) window.addEvent('load', inject);
				else inject();
			}
		} else {
			this.position = this.hide = this.show = this.dispose = Function.from(this);
		}
	},

	position: function(){
		if (!IframeShim.ready || !this.shim) return this;
		var size = this.element.measure(function(){
			return this.getSize();
		});
		if (this.options.margin != undefined){
			size.x = size.x - (this.options.margin * 2);
			size.y = size.y - (this.options.margin * 2);
			this.options.offset.x += this.options.margin;
			this.options.offset.y += this.options.margin;
		}
		this.shim.set({width: size.x, height: size.y}).position({
			relativeTo: this.element,
			offset: this.options.offset
		});
		return this;
	},

	hide: function(){
		if (this.shim) this.shim.setStyle('display', 'none');
		return this;
	},

	show: function(){
		if (this.shim) this.shim.setStyle('display', 'block');
		return this.position();
	},

	dispose: function(){
		if (this.shim) this.shim.dispose();
		return this;
	},

	destroy: function(){
		if (this.shim) this.shim.destroy();
		return this;
	}

});

window.addEvent('load', function(){
	IframeShim.ready = true;
});

